// WrapCheckers3D.java
// Andrew Davison, April 2005, ad@fivedots.coe.psu.ac.th

/* Illustrates how to create a simple world. The checkboard
 Refactored by Carl Weiman, 06-Feb-06
*/
package j3d.examples.checkers;

import com.sun.j3d.utils.behaviors.vp.OrbitBehavior;
import com.sun.j3d.utils.geometry.Box;
import com.sun.j3d.utils.geometry.Sphere;
import com.sun.j3d.utils.universe.SimpleUniverse;
import com.sun.j3d.utils.universe.ViewingPlatform;
import j3d.utils.RunTexCoordGenerationPanel;
import j3d.utils.Utils;
import utils.ResourceManager;

import javax.media.j3d.*;
import javax.swing.*;
import javax.vecmath.Color3f;
import javax.vecmath.Point3d;
import javax.vecmath.Vector3d;
import javax.vecmath.Vector3f;
import java.awt.*;
import java.io.File;

import gui.ClosableJFrame;


public class WrapCheckers3Dnew extends JPanel {
    private static final int screenWidth = 400;   // size of panel
    private static final int screenHeight = 400;
    TexCoordGeneration tcg = null;
    // initial user position
    private static final Point3d USERPOSN = new Point3d(0, 5, 20);
             Appearance redApp = new Appearance();
    // Palette
    Color3f black = new Color3f(0.0f, 0.0f, 0.0f);
    Color3f red = new Color3f(0.8f, 0.3f, 0.2f);
    Color3f green = new Color3f(0.2f, 0.8f, 0.2f);
    Color3f blue = new Color3f(0.2f, 0.2f, 0.8f);
    Color3f pink = new Color3f(0.9f, 0.4f, 0.4f);
    Color3f white = new Color3f(1.0f, 1.0f, 1.0f);
    Color3f gray = new Color3f(.4f, .4f, .4f);
    Color3f blueSky = new Color3f(0.17f, 0.65f, 0.92f);
    Color3f specular = new Color3f(0.9f, 0.9f, 0.9f);

    Material redMat = new Material(red, black, red, specular, 100.0f);
    Material greenMat = new Material(green, black, green, specular, 100.0f);

    Vector3f leftBackLight = new Vector3f(-1.0f, -1.0f, -1.0f);
    // left, down, backwards
    Vector3f rightFrontLight = new Vector3f(1.0f, -1.0f, 1.0f);
    // right, down, forwards

    private SimpleUniverse su;
    private BoundingSphere boundingSphere;  // for environment nodes

    private double boundingSphereRadius = 10;

    public WrapCheckers3Dnew() {
        // A JPanel holding a 3D canvas: linking Java 3D to Swing
        this.setLayout(new BorderLayout()); // "this" used to illustrate superclass methods
        this.setPreferredSize(new Dimension(screenWidth, screenHeight));
        Canvas3D canvas3D = new Canvas3D(SimpleUniverse.getPreferredConfiguration());
        this.add(canvas3D, BorderLayout.CENTER); // Put canvas on JPanel
        canvas3D.setFocusable(true); // enable canvas to focus
        canvas3D.requestFocus();     // ready for keys and mouse events

        su = new SimpleUniverse(canvas3D); // create universe with canvas as window
        boundingSphere = new BoundingSphere(new Point3d(0, 0, 0), boundingSphereRadius);

        su.addBranchGraph(createSceneGraph());

        initUserPosition();        // set user's viewpoint
        orbitControls(canvas3D);   // controls for moving the viewpoint

    } // end of WrapCheckers3D()


    private BranchGroup createSceneGraph() {
        BranchGroup sceneBranchGroup = new BranchGroup();

        // add the lights
        sceneBranchGroup.addChild(setDirectionalLight(blue, leftBackLight));
        sceneBranchGroup.addChild(setDirectionalLight(green, rightFrontLight));
        sceneBranchGroup.addChild(setAmbientLight(new Color3f(.9f, .2f, .3f)));
        sceneBranchGroup.addChild(setSceneBackground(blueSky));   // add the sky

        // add the geometry
        sceneBranchGroup.addChild(new CheckerFloor().getBG()); // add the floor
        // Add spheres at (position, radius)
        sceneBranchGroup.addChild(getSphere(new Point3d(1, 1, 1), 1));
        // sceneBranchGroup.addChild(getSphere(new Point3d(-2, -2, -4), 3));
        sceneBranchGroup.addChild(getSphere(2.f, .5f)); // y-position only
        sceneBranchGroup.addChild(getBox(new Point3d(-2, -2, -4), 1f, 2.f, .5f)); // y-position only

        sceneBranchGroup.compile();   // concatenate matrix transformations
        return sceneBranchGroup;
    } // end of createSceneGraph()

    private TransformGroup getSphere(float y, float r) {
        return (this.getSphere(new Point3d(0, y, 0), r));
    }

    private TransformGroup getBox(Point3d p3d, float h, float w, float d) {
        // Create the red appearance node
        // sets ambient, emissive, diffuse, specular, shininess
        greenMat.setLightingEnable(true);

        Appearance greenApp = new Appearance();
        greenApp.setMaterial(greenMat);

        redMat.setLightingEnable(true);
        redApp.setMaterial(redMat);

        // position the sphere
        Transform3D t3d = new Transform3D();
        t3d.set(new Vector3f(p3d));

        TransformGroup tg = new TransformGroup(t3d);
        // Argument 3 is number of divisions

        Box box = new Box(2f, 3f, 4f, Box.GENERATE_TEXTURE_COORDS, greenApp);
        Shape3D back = box.getShape(Box.BACK);
        back.setAppearance(redApp);
        Shape3D front = box.getShape(Box.FRONT);
        //  front.setAppearance(new Image);
        ResourceManager rm = ResourceManager.getResourceManager();
        File f = rm.getImageFile("earth.jpg");
        Texture texImage = Utils.getTexture(f);
        redApp.setTexture(texImage);
        setCapabilityBits(redApp);
        RunTexCoordGenerationPanel rtcgp = new RunTexCoordGenerationPanel() {
            public void run() {
                redApp.setTexCoordGeneration(getValue());
            }
        };
        ClosableJFrame cf = new ClosableJFrame();
        cf.getContentPane().add(rtcgp);
        cf.setSize(400,400);
        cf.show();
        front.setAppearance(redApp);
        tg.addChild(box);   // set its radius and appearance

        //  tg.addChild(new Sphere(radius, redApp));   // set its radius and appearance
        return tg;
    }

    private TransformGroup getSphere(Point3d p3d, float radius) {
        Sphere s = new Sphere();
        redMat.setLightingEnable(true);

        Appearance redApp = new Appearance();
        redApp.setMaterial(redMat);

        // position the sphere
        Transform3D t3d = new Transform3D();
        t3d.set(new Vector3f(p3d));

        TransformGroup tg = new TransformGroup(t3d);
        // Argument 3 is number of divisions
        tg.addChild(new Sphere(radius, 1, 50, redApp));   // set its radius and appearance

        //  tg.addChild(new Sphere(radius, redApp));   // set its radius and appearance
        return tg;
    }


    private AmbientLight setAmbientLight(Color3f colorAmbient) {

        // Set up the ambient light
        AmbientLight ambientLightNode = new AmbientLight(colorAmbient);
        ambientLightNode.setInfluencingBounds(boundingSphere);

        return ambientLightNode;
    }  // end of lightScene()


    private DirectionalLight setDirectionalLight
            (Color3f colorDirectional, Vector3f direction) {
        DirectionalLight light =
                new DirectionalLight(colorDirectional, direction);
        light.setInfluencingBounds(boundingSphere);
        return light;
    }

    private Background setSceneBackground(Color3f color) { // e.g., sky
        Background back = new Background();
        back.setApplicationBounds(boundingSphere);
        back.setColor(color);    // sky color
        return back;
    }  // end of addBackground()


    private void orbitControls(Canvas3D c) { //OrbitBehaviour allows
        //  user to rotate around the scene, and zoom in and out.
        OrbitBehavior orbit =
                new OrbitBehavior(c, OrbitBehavior.REVERSE_ALL);
        orbit.setSchedulingBounds(boundingSphere);
        ViewingPlatform vp = su.getViewingPlatform();
        vp.setViewPlatformBehavior(orbit);
    }  // end of orbitControls()


    private void initUserPosition()
    // Set the user's initial viewpoint using lookAt()
    {
        ViewingPlatform vp = su.getViewingPlatform();
        TransformGroup steerTG = vp.getViewPlatformTransform();

        Transform3D t3d = new Transform3D();
        steerTG.getTransform(t3d);
        // args are: viewer posn, where looking, up direction
        t3d.lookAt(USERPOSN, new Point3d(0, 0, 0), new Vector3d(0, 1, 0));
        t3d.invert();

        steerTG.setTransform(t3d);
    }  // end of initUserPosition()

    private void setCapabilityBits(Appearance app) {
        app.setCapability(Appearance.ALLOW_POLYGON_ATTRIBUTES_READ);
        app.setCapability(Appearance.ALLOW_POLYGON_ATTRIBUTES_WRITE);
        app.setCapability(Appearance.ALLOW_RENDERING_ATTRIBUTES_READ);
        app.setCapability(Appearance.ALLOW_TEXGEN_READ);
        app.setCapability(Appearance.ALLOW_TEXGEN_WRITE);
    }

} // end of WrapCheckers3Dnew class